/***********************************************************
Provides link to evolution algrithm functionality

William Westrick
Nicole Rodia
************************************************************/

#define NUM_PHENOTYPES 25
#define PI 3.141592

//40% probability of mutation
#define MUTATION_RATE (RAND_MAX*0.4) 

#define FITNESS_THRESHOLD 100

// for simulation of omnidrive operation: 
#define A ((float)(sin(PI/3)*3.0))
#define B ((float)(sin(PI/3)*2.0))
#define C ((float)(cos(PI/3)*3.0))
#define D ((float)(-cos(PI/3)*2.0))
#define E ((float)(-5.0))
#define F ((float)(-2.0))
#define G ((float)(10.0))
#define H ((float)(-5.0))


/***********************************************************
* VARIABLES
************************************************************/

//create a union to represent 4 bytes
typedef union {
	unsigned long value;
	struct {
		unsigned char low_byte;
		unsigned char byte_2;
		unsigned char byte_3;
		unsigned char high_byte;
	};
	signed char chars[4];
} four_chars_t;

//create a structure to hold the phenotypes
typedef struct {
	four_chars_t motors;
	unsigned long fitness;
} phenotype;

//population of phenotypes
phenotype population[NUM_PHENOTYPES];

//the current phenotype that is being updated
phenotype newPhenotype;

//define a structure for use with omnidrive simulation
typedef struct {
	unsigned char PWMvalue;
	unsigned char Direction;
	signed char totalvalue;
} motor;

//variables to represent the motors for simulation
motor motoroutputs[3];

//variable to hold the three position variables after movement
signed long Position[3];

//statistics variables
unsigned long WorstFitness;
unsigned long displayavgfitness;
unsigned long avgfitness;
unsigned long minfitness;

//variables to represent the current epoch and individual
unsigned char epoch;
unsigned char curMember;

//status variables
unsigned char hasInitialized;
unsigned char runningMember;
unsigned char doneRunningMember;

char temp_str[50]; //temporary string for serial communications

/***********************************************************
* FUNCTION PROTOTYPES
************************************************************/
//initialize the population
void initializePopulation(void);
//extract the values from phenotypes and assign to motors
void SetOutput(unsigned char num, signed char value);
//run the current phenotype
void runPhenotype(phenotype individual);
//stop the current phenotype
void stopPhenotype(void);
//calculate the result from the motion
void CalculateResult(void);
//find the fitness value for the last motion
unsigned long DetermineFitness(signed long DesiredX,signed long DesiredY,signed long DesiredTheta);
//breed an individual
phenotype BreedPhenotype(phenotype pheno);
//mutate a phenotype
phenotype mutatePhenotype(phenotype pheno); 
//initialize the evolution algorithm
void initializeEvolution(void);
//start a new epoch
void StartNewEpoch(void);
//start running the next member
void StartNextMember(void);
//see if the current phenotype should be updated
void UpdateMember(void);
//manage the evolutionary process
unsigned char ManageEvolution(void);

