/***********************************************************
Code to manage PWM generation for 3 motors

William Westrick
Nicole Rodia
************************************************************/

#include "motor_pwm.h"

/***********************************************************
* void initializeMotors(void)
* 		Sets up the PWM generation routines and counters
************************************************************/
void initializeMotors(void)
{
	//setup the motor variables
	motor1_PWM=0;
	motor1_DIR=MOTOR1_FWD;
	motor2_PWM=0;
	motor2_DIR=MOTOR2_FWD;
	motor3_PWM=0;
	motor3_DIR=MOTOR3_FWD;

	PWM_time=MAX_PWM_TIME;
}

/***********************************************************
* void SetOutput(unsigned char, unsigned char)
* 		Sets the PWM generation variables appropriately
************************************************************/
void SetOutput(unsigned char num, signed char value)
{
	//set the motors appropriately
	if (value>=0)
	{
		//make sure the value is clipped to the max
		if (value>100)
		{
			value=100;
		}

		//select the motor appropriately
		// and set the PWM value and direction
		switch (num)
		{
		case 0:
			motor1_PWM=value;
			motor1_DIR=MOTOR1_FWD;
			break;
		case 1:
			motor2_PWM=value;
			motor2_DIR=MOTOR2_FWD;
			break;
		case 2:
			motor3_PWM=value;
			motor3_DIR=MOTOR3_FWD;
			break;
		}
	} else {
		//negative direction, set the value and direction
		value = -value;
		if (value>100)
		{
			value=100;
		}

		switch (num)
		{
		case 0:
			motor1_PWM=value;
			motor1_DIR=MOTOR1_REV;
			break;
		case 1:
			motor2_PWM=value;
			motor2_DIR=MOTOR2_REV;
			break;
		case 2:
			motor3_PWM=value;
			motor3_DIR=MOTOR3_REV;
			break;
		}
	}
}
